/*
bhdev.h
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#ifndef _BH_POSTDEV_
#define _BH_POSTDEV_

#include "config.h"
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <termios.h>
#include <unistd.h>
#include <sigc++/slot.h>
#include <glibmm/main.h>
#include <glibmm/ustring.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include <usb.h>

/**
 * @defgroup mflib MF Libraries
 * @{
 */
 
namespace bhpos{
   
/**
 * @defgroup bhdev Devices 
 * @{
 *
 * \ingroup commlib
 * @{
 */

/*! \brief Notification Types. */
enum BhDevNotifyType{
	BHDEV_NOTIFY_TIMEOUT  = 1,
	BHDEV_NOTIFY_READ     = 2,
	BHDEV_NOTIFY_WRITE    = 4,
   BHDEV_NOTIFY_PRIORITY = 8,
   BHDEV_NOTIFY_HUP      = 16,
	BHDEV_NOTIFY_ERROR    = 32
};

/*! \brief Device Parameters */
enum BhDevParam{
	BHDEV_PARAM_TYPE,
   BHDEV_PARAM_DEVICE,
	BHDEV_PARAM_ENABLE,
	BHDEV_PARAM_TOENABLE,
	BHDEV_PARAM_TOVALUE,
	BHDEV_PARAM_TORETRIES,
	BHDEV_PARAM_TOBACKOFF,
	BHDEV_PARAM_TOBOVALUE,
	BHDEV_PARAM_INPUTBUFFSIZE,
	BHDEV_PARAM_OUTPUTBUFFSIZE,
	BHDEV_PARAM_SERIALSPEED,
	BHDEV_PARAM_SERIALBITS,
	BHDEV_PARAM_SERIALSTOPBITS,
	BHDEV_PARAM_SERIALPARITY,
	BHDEV_PARAM_SERIALSOFTFLOW,
	BHDEV_PARAM_SERIALBITSET,
	BHDEV_PARAM_SERIALBITCLR,
	BHDEV_PARAM_SERIALBITGET
};

/*! \brief Serial Device Parity */
enum BhDevSerialParity{
   BHDEV_SERIAL_PARITY_NONE,
   BHDEV_SERIAL_PARITY_ODD,
   BHDEV_SERIAL_PARITY_EVEN
};

/*! \brief Serial Device Control */
enum BhDevSerialControl{
   BHDEV_SERIAL_CTRL_DTR,
   BHDEV_SERIAL_CTRL_RTS,
   BHDEV_SERIAL_CTRL_CTS,
   BHDEV_SERIAL_CTRL_DSR,
   BHDEV_SERIAL_CTRL_CD,
   BHDEV_SERIAL_CTRL_RI,
   BHDEV_SERIAL_CTRL_RX,
   BHDEV_SERIAL_CTRL_TX
};

/*! \brief Device Errors */
enum BhDevError{
	BHDEV_ERR_NONE=0,
	BHDEV_ERR_DEVNULLPARAM,
	BHDEV_ERR_DEVNAMEALLOC,
	BHDEV_ERR_DEVOPEN,
	BHDEV_ERR_TIMEOUT=100,
	BHDEV_ERR_READ,
	BHDEV_ERR_WRITE
};

/*! \brief Devices Types */
enum BhDevType{
   BHDEV_TYPE_NONE=0,
	BHDEV_TYPE_DESCRIPTOR,
	BHDEV_TYPE_SERIAL,
	BHDEV_TYPE_PARALLEL,
	BHDEV_TYPE_USB,
	BHDEV_TYPE_NETWORK,
	BHDEV_TYPE_FIREWIRE
};

#ifdef _USE_USB
#define READ_FROM_USB               0x45
#define WRITE_TO_USB                0x46
#define START_USB_COMM              0x47
#define END_USB_COMM                0x48
#define BULK_READ_ENDPOINT          0x82
#define BULK_WRITE_ENDPOINT         0x01
#endif

#define READ_NORMAL     0
#define READ_PRIORITY   1
#define ERROR_ERROR     0
#define ERROR_HUP       1
#define ERROR_NVAL      2

/*! \brief A class to handle devices. */
class CBhDevice : public sigc::trackable
{
private:
	Glib::ustring devname;
	BhDevType     type;
	bool	 bEnabled;
   usb_dev_handle  *usb_dev;
	bool	 bTimeout;
	int	 timeoutval;
	bool	 bRetry;
	int	 retries;
	int	 max_retries;
	bool	 bBackoff;
	double backoff_value;

protected:	
   int	 fd;
	BhDevError      bherror;
	int 		       error;
	BhDevNotifyType notifytypes;

	struct termios old_tio;
   
   sigc::connection connection;
   sigc::connection timeout;
   
   sigc::signal<void> write_signal;
   sigc::signal<void, int> read_signal;
   sigc::signal<void, int> error_signal;
   sigc::signal<void> idle_signal;
   sigc::signal<void> timeout_signal;
   sigc::signal<void> monitor_signal;

   bool Active(Glib::IOCondition);
   bool Timeout();
   void SetSignals();
	
	int privopen();
   
   void usbClose();
   int  usbFind(int type);
   int  usbWrite(int flags, int len, void *data);
   int  usbRead(int flags, int len, void *data);
   
public:
	CBhDevice(BhDevType);
	virtual ~CBhDevice();

   void operator=(CBhDevice const&);
	bool operator==(CBhDevice const&);
   bool operator!=(CBhDevice const&);

   /*! \brief Virtual functions R us */
	virtual int  Open(Glib::ustring, BhDevNotifyType);
	virtual void Close();
	virtual int  Read(int, int, void *);
	virtual int  Write(int, int, void *);
   virtual int  Control(int);
   virtual int  Monitor(int, int, void *);
   virtual int  Status(int);
	virtual void SetParameter(BhDevParam, void *);
	virtual void GetParameter(BhDevParam, void *);
	
	int   GetError();
	char *GetErrorString(int err);

   sigc::signal<void> signal_write() { return write_signal; }
   sigc::signal<void, int> signal_read() { return read_signal; }
   sigc::signal<void, int> signal_error() { return error_signal; }
   sigc::signal<void> signal_idle() { return idle_signal; }
   sigc::signal<void> signal_timeout() { return timeout_signal; }
   sigc::signal<void> signal_monitor() { return monitor_signal; }
};

namespace parser{
/*! \brief A class to handle device config parsing. */
class CDeviceParser
{
protected:
	char		 *name;
	char		 *manufacturer;
	char		 *model;
	char 		*filename;
	char 		*protocol;
	xmlNsPtr	ns;
	
	int		  error;
	char      *device;
	int	     type;
	
public:
	CDeviceParser();
	~CDeviceParser();
	int   ParseDevice(xmlDocPtr, xmlNsPtr, xmlNodePtr, CBhDevice&);
	int   SaveDevice(xmlDocPtr, xmlNsPtr, xmlNodePtr, CBhDevice&);
	void  ParseOptions(xmlNodePtr, CBhDevice&);
	int   GetError();
	char *GetErrorString(int);
};

/* @} 
	@} */

}

/* @} */

}

#endif // _BH_POSTDEV_
